import {
  AudioPlayerStatus,
  createAudioPlayer,
  createAudioResource,
  VoiceConnectionStatus,
} from "@discordjs/voice";
import { Command, Extras } from ".";
import { existsSync } from "node:fs";
import { Duplex, Readable } from "node:stream";
import ytdl from "@distube/ytdl-core";

const cmdPlay: Command = {
  names: ["play"],
  desc: "test voice features",
  splitArgs: true,

  async checkPerm(msg) {
    return msg.author.id === "778441081883983893";
  },

  async handler(ex: Extras, ...args: string[]) {
    const voiceState = ex.msg.member?.voiceState;
    if (!voiceState?.channelID)
      return await ex.reply("You are not in a voice channel");

    if (
      ex.msg.guildID !== null &&
      ex.msg.client.getVoiceConnection(ex.msg.guildID)
    ) {
      return await ex.reply("I am already connected");
    }

    const conn = voiceState.channel?.join({
      selfMute: false,
    });
    if (!conn) return await ex.reply("Failed to join channel");

    conn.on(VoiceConnectionStatus.Disconnected, () => {
      console.log("Rejoining from disconnection!!");
      conn.rejoin();
    });

    const player = createAudioPlayer();
    conn.subscribe(player);

    conn.on("stateChange", (oldSt, newSt) => {
      console.log(`Moving from ${oldSt.status} to ${newSt.status}`);
    });
    conn.on("error", (err) => {
      console.error("Connection experienced error!", err);
    });

    player.once(AudioPlayerStatus.Playing, async () => {
      await ex.say("Playing audio");
    });
    player.on(AudioPlayerStatus.Idle, async () => {
      await ex.say("Audio finished playing");
    });

    const url = args[0];
    if (!url) {
      return await ex.reply("No url was specified");
    }

    const readable = ytdl(url, {
      filter: "audioonly",
      quality: "highestaudio",
      playerClients: ["IOS", "ANDROID"],
    });

    const audio = createAudioResource(readable);
    player.play(audio);
  },
};

export default cmdPlay;
