import { Client } from "oceanic.js";
import "dotenv/config";
import commands, { Extras } from "./commands";
import { PREFIX, SPLIT } from "./constants";

function token() {
  return (
    process.env.DISCORD_TOKEN ??
    (() => {
      console.error("No `DISCORD_TOKEN` was specified!!");
      process.exit(1);
    })()
  );
}

const client = new Client({
  auth: `Bot ${token()}`,
  gateway: {
    intents: ["ALL"],
  },
});

// --

client.on("ready", async () => {
  console.log("Ready as", client.user.tag);
});

client.on("messageCreate", async (msg) => {
  if (msg.author.bot) return;

  const scanContent = msg.content.toLowerCase();
  if (!scanContent.startsWith(PREFIX)) return;
  const cmdName = scanContent.split(SPLIT)[0]?.slice(PREFIX.length);

  if (cmdName === undefined) return;

  const cmd = commands.find((cmd) => cmd.names.includes(cmdName));

  if (cmd === undefined) return;

  if (await cmd.checkPerm(msg)) {
    const ex = new Extras(msg);

    const args = msg.content.split(SPLIT);
    args.shift();

    if (cmd.splitArgs) await cmd.handler(ex, ...args);
    else {
      const content = msg.content.slice(PREFIX.length + cmdName.length);
      await cmd.handler(ex, content);
    }
  } else await msg.createReaction("❌");
});

client.on("error", (err) => {
  console.error("Error!!", err);
});

client.connect();
